@@ -70,14 +70,6 @@ typedef struct
     Scheme_Object   *port;
 } Port_Info;
 
-/* info for do_apply */
-typedef struct
-{
-    Scheme_Object   *proc;
-    int		    argc;
-    Scheme_Object   **argv;
-} Apply_Info;
-
 /*
  *========================================================================
  *  Vim-Control Commands
@@ -160,7 +152,6 @@ static Scheme_Object *_apply_thunk_catch_exceptions(
 static Scheme_Object *extract_exn_message(Scheme_Object *v);
 static Scheme_Object *do_eval(void *, int noargc, Scheme_Object **noargv);
 static Scheme_Object *do_load(void *, int noargc, Scheme_Object **noargv);
-static Scheme_Object *do_apply(void *, int noargc, Scheme_Object **noargv);
 static void register_vim_exn(void);
 static vim_mz_buffer *get_buffer_arg(const char *fname, int argnum,
 	int argc, Scheme_Object **argv);
@@ -635,8 +626,10 @@ dynamic_mzscheme_end(void)
 #endif /* DYNAMIC_MZSCHEME */
 
 /* need to put it here for dynamic stuff to work */
-#ifdef INCLUDE_MZSCHEME_BASE
+#if defined(INCLUDE_MZSCHEME_BASE)
 # include "mzscheme_base.c"
+#elif MZSCHEME_VERSION_MAJOR >= 400
+# error MzScheme 4.x must include mzscheme_base.c, for MinGW32 you need to define MZSCHEME_GENERATE_BASE=yes
 #endif
 
 /*
@@ -875,14 +868,14 @@ startup_mzscheme(void)
 #ifdef INCLUDE_MZSCHEME_BASE
     {
 	/*
-	 * versions 4.x do not provide Scheme bindings by defaults
+	 * versions 4.x do not provide Scheme bindings by default
 	 * we need to add them explicitly
 	 */
 	Scheme_Object *scheme_base_symbol = NULL;
 	MZ_GC_DECL_REG(1);
 	MZ_GC_VAR_IN_REG(0, scheme_base_symbol);
 	MZ_GC_REG();
-	/* invoke function from generated and included base.c */
+	/* invoke function from generated and included mzscheme_base.c */
 	declare_modules(environment);
 	scheme_base_symbol = scheme_intern_symbol("scheme/base");
 	MZ_GC_CHECK();
@@ -1054,7 +1047,7 @@ mzscheme_init(void)
 	MZ_GC_REG();
 	config = scheme_config;
 	MZ_GC_CHECK();
-	/* recreate ports each call effectivelly clearing these ones */
+	/* recreate ports each call effectively clearing these ones */
 	curout = scheme_make_string_output_port();
 	MZ_GC_CHECK();
 	curerr = scheme_make_string_output_port();
@@ -1194,36 +1187,6 @@ ex_mzscheme(exarg_T *eap)
     }
 }
 
-/*
- * apply MzScheme procedure with arguments,
- * handling errors
- */
-    Scheme_Object *
-mzvim_apply(Scheme_Object *proc, int argc, Scheme_Object **argv)
-{
-    if (mzscheme_init())
-	return FAIL;
-    else
-    {
-	Apply_Info	data = {NULL, 0, NULL};
-	Scheme_Object	*ret = NULL;
-
-	MZ_GC_DECL_REG(5);
-	MZ_GC_VAR_IN_REG(0, ret);
-	MZ_GC_VAR_IN_REG(1, data.proc);
-	MZ_GC_ARRAY_VAR_IN_REG(2, data.argv, argc);
-	MZ_GC_REG();
-
-	data.proc = proc;
-	data.argc = argc;
-	data.argv = argv;
-
-	eval_with_exn_handling(&data, do_apply, &ret);
-	MZ_GC_UNREG();
-	return ret;
-    }
-}
-
     static Scheme_Object *
 do_load(void *data, int noargc, Scheme_Object **noargv)
 {
@@ -1255,7 +1218,7 @@ do_load(void *data, int noargc, Scheme_Object **noargv)
 	MZ_GC_CHECK();
     }
 
-    /* errors will be caught in do_mzscheme_comamnd and ex_mzfile */
+    /* errors will be caught in do_mzscheme_command and ex_mzfile */
     scheme_close_input_port(pinfo->port);
     MZ_GC_CHECK();
     pinfo->port = NULL;
@@ -1346,13 +1309,6 @@ do_eval(void *s, int noargc, Scheme_Object **noargv)
     return scheme_eval_string_all((char *)s, environment, TRUE);
 }
 
-    static Scheme_Object *
-do_apply(void *a, int noargc, Scheme_Object **noargv)
-{
-    Apply_Info	*info = (Apply_Info *)a;
-    return scheme_apply(info->proc, info->argc, info->argv);
-}
-
 /*
  *========================================================================
  *  3.  MzScheme I/O Handlers
@@ -2126,7 +2082,7 @@ get_buffer_line_list(void *data, int argc, Scheme_Object **argv)
     static Scheme_Object *
 set_buffer_line(void *data, int argc, Scheme_Object **argv)
 {
-    /* First of all, we check the the of the supplied MzScheme object.
+    /* First of all, we check the value of the supplied MzScheme object.
      * There are three cases:
      *	  1. #f - this is a deletion.
      *	  2. A string	   - this is a replacement.
@@ -2426,7 +2382,7 @@ set_buffer_line_list(void *data, int argc, Scheme_Object **argv)
 /*
  * (insert-buff-line-list {linenr} {string/string-list} [buffer])
  *
- * Insert a number of lines into the specified buffer after the specifed line.
+ * Insert a number of lines into the specified buffer after the specified line.
  * The line number is in Vim format (1-based). The lines to be inserted are
  * given as an MzScheme list of string objects or as a single string. The lines
  * to be added are checked for validity and correct format. Errors are
