@@ -2,62 +2,86 @@
  *
  *  2.10.90 changed to word output
  *  3.03.93 new indent style, dumb bug inserted and fixed.
- *          -c option, mls
+ *	    -c option, mls
  * 26.04.94 better option parser, -ps, -l, -s added.
  *  1.07.94 -r badly needs - as input file.  Per default autoskip over
- *             consequtive lines of zeroes, as unix od does.
- *          -a shows them too.
- *          -i dump as c-style #include "file.h"
+ *	       consecutive lines of zeroes, as unix od does.
+ *	    -a shows them too.
+ *	    -i dump as c-style #include "file.h"
  *  1.11.95 if "xxd -i" knows the filename, an 'unsigned char filename_bits[]'
- *          array is written in correct c-syntax.
- *          -s improved, now defaults to absolute seek, relative requires a '+'.
- *          -r improved, now -r -s -0x... is supported.
- *             change/suppress leading '\0' bytes.
- *          -l n improved: stops exactly after n bytes.
- *          -r improved, better handling of partial lines with trailing garbage.
- *          -r improved, now -r -p works again!
- *          -r improved, less flushing, much faster now! (that was silly)
+ *	    array is written in correct c-syntax.
+ *	    -s improved, now defaults to absolute seek, relative requires a '+'.
+ *	    -r improved, now -r -s -0x... is supported.
+ *	       change/suppress leading '\0' bytes.
+ *	    -l n improved: stops exactly after n bytes.
+ *	    -r improved, better handling of partial lines with trailing garbage.
+ *	    -r improved, now -r -p works again!
+ *	    -r improved, less flushing, much faster now! (that was silly)
  *  3.04.96 Per repeated request of a single person: autoskip defaults to off.
  * 15.05.96 -v added. They want to know the version.
- *          -a fixed, to show last line inf file ends in all zeros.
- *          -u added: Print upper case hex-letters, as preferred by unix bc.
- *          -h added to usage message. Usage message extended.
- *          Now using outfile if specified even in normal mode, aehem.
- *          No longer mixing of ints and longs. May help doze people.
- *          Added binify ioctl for same reason. (Enough Doze stress for 1996!)
+ *	    -a fixed, to show last line inf file ends in all zeros.
+ *	    -u added: Print upper case hex-letters, as preferred by unix bc.
+ *	    -h added to usage message. Usage message extended.
+ *	    Now using outfile if specified even in normal mode, aehem.
+ *	    No longer mixing of ints and longs. May help doze people.
+ *	    Added binify ioctl for same reason. (Enough Doze stress for 1996!)
  * 16.05.96 -p improved, removed occasional superfluous linefeed.
  * 20.05.96 -l 0 fixed. tried to read anyway.
  * 21.05.96 -i fixed. now honours -u, and prepends __ to numeric filenames.
- *          compile -DWIN32 for NT or W95. George V. Reilly, * -v improved :-)
- *          support --gnuish-longhorn-options
+ *	    compile -DWIN32 for NT or W95. George V. Reilly, * -v improved :-)
+ *	    support --gnuish-longhorn-options
  * 25.05.96 MAC support added: CodeWarrior already uses ``outline'' in Types.h
- *          which is included by MacHeaders (Axel Kielhorn). Renamed to
- *          xxdline().
+ *	    which is included by MacHeaders (Axel Kielhorn). Renamed to
+ *	    xxdline().
  *  7.06.96 -i printed 'int' instead of 'char'. *blush*
- *          added Bram's OS2 ifdefs...
+ *	    added Bram's OS2 ifdefs...
  * 18.07.96 gcc -Wall @ SunOS4 is now slient.
- *          Added osver for MSDOS/DJGPP/WIN32.
+ *	    Added osver for MSDOS/DJGPP/WIN32.
  * 29.08.96 Added size_t to strncmp() for Amiga.
  * 24.03.97 Windows NT support (Phil Hanna). Clean exit for Amiga WB (Bram)
  * 02.04.97 Added -E option, to have EBCDIC translation instead of ASCII
- *          (antonio.colombo@jrc.org)
+ *	    (azc10@yahoo.com)
  * 22.05.97 added -g (group octets) option (jcook@namerica.kla.com).
+ * 23.09.98 nasty -p -r misfeature fixed: slightly wrong output, when -c was
+ *	    missing or wrong.
+ * 26.09.98 Fixed: 'xxd -i infile outfile' did not truncate outfile.
+ * 27.10.98 Fixed: -g option parser required blank.
+ *	    option -b added: 01000101 binary output in normal format.
+ * 16.05.00 Added VAXC changes by Stephen P. Wall
+ * 16.05.00 Improved MMS file and merge for VMS by Zoltan Arpadffy
  *
- * (c) 1990-1997 by Juergen Weigert (jnweiger@informatik.uni-erlangen.de)
+ * (c) 1990-1998 by Juergen Weigert (jnweiger@informatik.uni-erlangen.de)
+ *
+ * Small changes made afterwards by Bram Moolenaar et al.
  *
  * Distribute freely and credit me,
  * make money and share with me,
  * lose money and don't ask me.
  */
+
+/* Visual Studio 2005 has 'deprecated' many of the standard CRT functions */
+#if _MSC_VER >= 1400
+# define _CRT_SECURE_NO_DEPRECATE
+# define _CRT_NONSTDC_NO_DEPRECATE
+#endif
+#if !defined(CYGWIN) && (defined(CYGWIN32) || defined(__CYGWIN__) || defined(__CYGWIN32__))
+# define CYGWIN
+#endif
+
 #include <stdio.h>
-#include <fcntl.h>
+#ifdef VAXC
+# include <file.h>
+#else
+# include <fcntl.h>
+#endif
 #ifdef __TSC__
 # define MSDOS
 #endif
 #if !defined(OS2) && defined(__EMX__)
 # define OS2
 #endif
-#if defined(MSDOS) || defined(WIN32) || defined(OS2)
+#if defined(MSDOS) || defined(WIN32) || defined(OS2) || defined(__BORLANDC__) \
+  || defined(CYGWIN)
 # include <io.h>	/* for setmode() */
 #else
 # ifdef UNIX
@@ -67,13 +91,20 @@
 #include <stdlib.h>
 #include <string.h>	/* for strncmp() */
 #include <ctype.h>	/* for isalnum() */
-#if __MWERKS__ && !defined(__BEOS__)
+#if __MWERKS__ && !defined(BEBOX)
 # include <unix.h>	/* for fdopen() on MAC */
 #endif
 
-/*
- * This corrects the problem of missing prototypes for certain functions
- * in some GNU installations (e.g. SunOS 4.1.x).
+#if defined(__BORLANDC__) && __BORLANDC__ <= 0x0410 && !defined(fileno)
+/* Missing define and prototype grabbed from the BC 4.0 <stdio.h> */
+# define fileno(f)       ((f)->fd)
+FILE   _FAR *_Cdecl _FARFUNC fdopen(int __handle, char _FAR *__type);
+#endif
+
+
+/*  This corrects the problem of missing prototypes for certain functions
+ *  in some GNU installations (e.g. SunOS 4.1.x).
+ *  Darren Hiebert <darren@hmi.com> (sparc-sun-sunos4.1.3_U1/2.7.2.2)
  */
 #if defined(__GNUC__) && defined(__STDC__)
 # ifndef __USE_FIXED_PROTOTYPES__
@@ -108,7 +139,7 @@ extern void perror __P((char *));
 extern long int strtol();
 extern long int ftell();
 
-char version[] = "xxd V1.8 22may97 by Juergen Weigert";
+char version[] = "xxd V1.10 27oct98 by Juergen Weigert";
 #ifdef WIN32
 char osver[] = " (Win32)";
 #else
@@ -129,12 +160,27 @@ char osver[] = "";
 # define BIN_CREAT(yes) ((yes) ? (O_CREAT|O_BINARY) : O_CREAT)
 # define BIN_ASSIGN(fp, yes) setmode(fileno(fp), (yes) ? O_BINARY : O_TEXT)
 # define PATH_SEP '\\'
-#else
-# define BIN_READ(dummy)  "r"
-# define BIN_WRITE(dummy) "w"
-# define BIN_CREAT(dummy) O_CREAT
-# define BIN_ASSIGN(fp, dummy) fp
+#elif defined(CYGWIN)
+# define BIN_READ(yes)  ((yes) ? "rb" : "rt")
+# define BIN_WRITE(yes) ((yes) ? "wb" : "w")
+# define BIN_CREAT(yes) ((yes) ? (O_CREAT|O_BINARY) : O_CREAT)
+# define BIN_ASSIGN(fp, yes) ((yes) ? (void) setmode(fileno(fp), O_BINARY) : (void) (fp))
 # define PATH_SEP '/'
+#else
+# ifdef VMS
+#  define BIN_READ(dummy)  "r"
+#  define BIN_WRITE(dummy) "w"
+#  define BIN_CREAT(dummy) O_CREAT
+#  define BIN_ASSIGN(fp, dummy) fp
+#  define PATH_SEP ']'
+#  define FILE_SEP '.'
+# else
+#  define BIN_READ(dummy)  "r"
+#  define BIN_WRITE(dummy) "w"
+#  define BIN_CREAT(dummy) O_CREAT
+#  define BIN_ASSIGN(fp, dummy) fp
+#  define PATH_SEP '/'
+# endif
 #endif
 
 /* open has only to arguments on the Mac */
@@ -151,7 +197,8 @@ char osver[] = "";
 #endif
 
 #ifndef __P
-# ifdef __STDC__
+# if defined(__STDC__) || defined(MSDOS) || defined(WIN32) || defined(OS2) \
+		|| defined(__BORLANDC__)
 #  define __P(a) a
 # else
 #  define __P(a) ()
@@ -164,9 +211,9 @@ static void exit_with_usage __P((char *));
 static int huntype __P((FILE *, FILE *, FILE *, char *, int, int, long));
 static void xxdline __P((FILE *, char *, int));
 
-#define TRY_SEEK 	/* attempt to use lseek, or skip forward by reading */
+#define TRY_SEEK	/* attempt to use lseek, or skip forward by reading */
 #define COLS 256	/* change here, if you ever need more columns */
-#define LLEN (9 + 3*COLS + 2 + COLS)
+#define LLEN (11 + (9*COLS-1)/1 + COLS + 2)
 
 char hexxa[] = "0123456789abcdef0123456789ABCDEF", *hexx = hexxa;
 
@@ -174,6 +221,7 @@ char hexxa[] = "0123456789abcdef0123456789ABCDEF", *hexx = hexxa;
 #define HEX_NORMAL 0
 #define HEX_POSTSCRIPT 1
 #define HEX_CINCLUDE 2
+#define HEX_BITS 3		/* not hex a dump, but bits: 01111001 */
 
 static void
 exit_with_usage(pname)
@@ -183,13 +231,14 @@ char *pname;
   fprintf(stderr, "    or\n       %s -r [-s [-]offset] [-c cols] [-ps] [infile [outfile]]\n", pname);
   fprintf(stderr, "Options:\n");
   fprintf(stderr, "    -a          toggle autoskip: A single '*' replaces nul-lines. Default off.\n");
+  fprintf(stderr, "    -b          binary digit dump (incompatible with -p,-i,-r). Default hex.\n");
   fprintf(stderr, "    -c cols     format <cols> octets per line. Default 16 (-i: 12, -ps: 30).\n");
   fprintf(stderr, "    -E          show characters in EBCDIC. Default ASCII.\n");
   fprintf(stderr, "    -g          number of octets per group in normal output. Default 2.\n");
   fprintf(stderr, "    -h          print this summary.\n");
   fprintf(stderr, "    -i          output in C include file style.\n");
   fprintf(stderr, "    -l len      stop after <len> octets.\n");
-  fprintf(stderr, "    -ps         output in postscript continuous hexdump style.\n");
+  fprintf(stderr, "    -ps         output in postscript plain hexdump style.\n");
   fprintf(stderr, "    -r          reverse operation: convert (or patch) hexdump into binary.\n");
   fprintf(stderr, "    -r -s off   revert with <off> added to file positions found in hexdump.\n");
   fprintf(stderr, "    -s %sseek  start at <seek> bytes abs. %sinfile offset.\n",
@@ -225,20 +274,26 @@ long base_off;
   while ((c = getc(fpi)) != EOF)
     {
       if (c == '\r')	/* Doze style input file? */
-        continue;
+	continue;
+
+#if 0	/* this doesn't work when there is normal text after the hex codes in
+	   the last line that looks like hex */
+      if (c == ' ' || c == '\n' || c == '\t')  /* allow multiple spaces */
+	continue;
+#endif
 
       n3 = n2;
       n2 = n1;
 
       if (c >= '0' && c <= '9')
-        n1 = c - '0';
+	n1 = c - '0';
       else if (c >= 'a' && c <= 'f')
-        n1 = c - 'a' + 10;
+	n1 = c - 'a' + 10;
       else if (c >= 'A' && c <= 'F')
-        n1 = c - 'A' + 10;
+	n1 = c - 'A' + 10;
       else
-        {
-          n1 = -1;
+	{
+	  n1 = -1;
 	  if (ign_garb)
 	    continue;
 	}
@@ -262,7 +317,7 @@ long base_off;
 	}
 
       if (base_off + want_off != have_off)
-        {
+	{
 	  fflush(fpo);
 #ifdef TRY_SEEK
 	  c = fseek(fpo, base_off + want_off - have_off, 1);
@@ -279,22 +334,23 @@ long base_off;
 	}
 
       if (n2 >= 0 && n1 >= 0)
-        {
+	{
 	  putc((n2 << 4) | n1, fpo);
 	  have_off++;
 	  want_off++;
 	  n1 = -1;
-	  if (++p >= cols)
+	  if ((++p >= cols) && !hextype)
 	    {
-	      if (!hextype)
-	        want_off = 0;
+	      /* skip rest of line as garbage */
+	      want_off = 0;
 	      while ((c = getc(fpi)) != '\n' && c != EOF)
-	        ;
+		;
 	      ign_garb = 1;
 	    }
 	}
       else if (n1 < 0 && n2 < 0 && n3 < 0)
-        {
+	{
+	  /* already stumbled into garbage, skip line, wait and see */
 	  if (!hextype)
 	    want_off = 0;
 	  while ((c = getc(fpi)) != '\n' && c != EOF)
@@ -347,7 +403,7 @@ int nz;
 	    fputs("*\n", fp);
 	}
       if (nz >= 0 || zero_seen > 0)
-        fputs(l, fp);
+	fputs(l, fp);
       if (nz)
 	zero_seen = 0;
     }
@@ -392,7 +448,7 @@ char *argv[];
   int c, e, p = 0, relseek = 1, negseek = 0, revert = 0;
   int cols = 0, nonzero = 0, autoskip = 0, hextype = HEX_NORMAL;
   int ebcdic = 0;
-  int octspergrp = 2;	/* number of octets grouped in output */
+  int octspergrp = -1;	/* number of octets grouped in output */
   int grplen;		/* total chars per octet group */
   long length = -1, n = 0, seekoff = 0;
   char l[LLEN+1];
@@ -408,18 +464,27 @@ char *argv[];
   for (pp = pname; *pp; )
     if (*pp++ == PATH_SEP)
       pname = pp;
+#ifdef FILE_SEP
+  for (pp = pname; *pp; pp++)
+    if (*pp == FILE_SEP)
+      {
+	*pp = '\0';
+	break;
+      }
+#endif
 
   while (argc >= 2)
     {
       pp = argv[1] + (!STRNCMP(argv[1], "--", 2) && argv[1][2]);
-           if (!STRNCMP(pp, "-a", 2)) autoskip = 1 - autoskip;
+	   if (!STRNCMP(pp, "-a", 2)) autoskip = 1 - autoskip;
+      else if (!STRNCMP(pp, "-b", 2)) hextype = HEX_BITS;
       else if (!STRNCMP(pp, "-u", 2)) hexx = hexxa + 16;
       else if (!STRNCMP(pp, "-p", 2)) hextype = HEX_POSTSCRIPT;
       else if (!STRNCMP(pp, "-i", 2)) hextype = HEX_CINCLUDE;
       else if (!STRNCMP(pp, "-r", 2)) revert++;
       else if (!STRNCMP(pp, "-E", 2)) ebcdic++;
       else if (!STRNCMP(pp, "-v", 2))
-        {
+	{
 	  fprintf(stderr, "%s%s\n", version, osver);
 	  exit(0);
 	}
@@ -430,19 +495,24 @@ char *argv[];
 	  else
 	    {
 	      if (!argv[2])
-	        exit_with_usage(pname);
+		exit_with_usage(pname);
 	      cols = (int)strtol(argv[2], NULL, 0);
 	      argv++;
 	      argc--;
 	    }
 	}
       else if (!STRNCMP(pp, "-g", 2))
-        {
-	  if (!argv[2])
-	    exit_with_usage(pname);
-	  octspergrp = (int)strtol(argv[2], NULL, 0);
-	  argv++;
-	  argc--;
+	{
+	  if (pp[2] && STRNCMP("group", pp + 2, 5))
+	    octspergrp = (int)strtol(pp + 2, NULL, 0);
+	  else
+	    {
+	      if (!argv[2])
+		exit_with_usage(pname);
+	      octspergrp = (int)strtol(argv[2], NULL, 0);
+	      argv++;
+	      argc--;
+	    }
 	}
       else if (!STRNCMP(pp, "-s", 2))
 	{
@@ -452,21 +522,21 @@ char *argv[];
 	    {
 #ifdef TRY_SEEK
 	      if (pp[2] == '+')
-	        relseek++;
+		relseek++;
 	      if (pp[2+relseek] == '-')
-	        negseek++;
+		negseek++;
 #endif
 	      seekoff = strtol(pp + 2+relseek+negseek, (char **)NULL, 0);
 	    }
 	  else
 	    {
 	      if (!argv[2])
-	        exit_with_usage(pname);
+		exit_with_usage(pname);
 #ifdef TRY_SEEK
 	      if (argv[2][0] == '+')
-	        relseek++;
+		relseek++;
 	      if (argv[2][relseek] == '-')
-	        negseek++;
+		negseek++;
 #endif
 	      seekoff = strtol(argv[2] + relseek+negseek, (char **)NULL, 0);
 	      argv++;
@@ -480,37 +550,49 @@ char *argv[];
 	  else
 	    {
 	      if (!argv[2])
-	        exit_with_usage(pname);
+		exit_with_usage(pname);
 	      length = strtol(argv[2], (char **)NULL, 0);
 	      argv++;
 	      argc--;
 	    }
 	}
       else if (!strcmp(pp, "--"))	/* end of options */
-        {
+	{
 	  argv++;
 	  argc--;
 	  break;
 	}
       else if (pp[0] == '-' && pp[1])	/* unknown option */
-        exit_with_usage(pname);
+	exit_with_usage(pname);
       else
-        break;				/* not an option */
+	break;				/* not an option */
 
       argv++;				/* advance to next argument */
       argc--;
     }
-  
+
   if (!cols)
     switch (hextype)
       {
-      case HEX_POSTSCRIPT: 	cols = 30; break;
+      case HEX_POSTSCRIPT:	cols = 30; break;
       case HEX_CINCLUDE:	cols = 12; break;
+      case HEX_BITS:		cols = 6; break;
       case HEX_NORMAL:
       default:			cols = 16; break;
       }
 
-  if (cols < 1 || (!hextype && (cols > COLS)))
+  if (octspergrp < 0)
+    switch (hextype)
+      {
+      case HEX_BITS:		octspergrp = 1; break;
+      case HEX_NORMAL:		octspergrp = 2; break;
+      case HEX_POSTSCRIPT:
+      case HEX_CINCLUDE:
+      default:			octspergrp = 0; break;
+      }
+
+  if (cols < 1 || ((hextype == HEX_NORMAL || hextype == HEX_BITS)
+							    && (cols > COLS)))
     {
       fprintf(stderr, "%s: invalid number of columns (max. %d).\n", pname, COLS);
       exit(1);
@@ -539,9 +621,10 @@ char *argv[];
   else
     {
       int fd;
+      int mode = revert ? O_WRONLY : (O_TRUNC|O_WRONLY);
 
-      if (((fd = OPEN(argv[2], O_WRONLY | BIN_CREAT(revert), 0666)) < 0) ||
-          (fpo = fdopen(fd, BIN_WRITE(revert))) == NULL)
+      if (((fd = OPEN(argv[2], mode | BIN_CREAT(revert), 0666)) < 0) ||
+	  (fpo = fdopen(fd, BIN_WRITE(revert))) == NULL)
 	{
 	  fprintf(stderr, "%s: ", pname);
 	  perror(argv[2]);
@@ -553,35 +636,35 @@ char *argv[];
   if (revert)
     {
       if (hextype && (hextype != HEX_POSTSCRIPT))
-        {
-          fprintf(stderr, "%s: sorry, cannot revert this type of hexdump\n", pname);
+	{
+	  fprintf(stderr, "%s: sorry, cannot revert this type of hexdump\n", pname);
 	  return -1;
 	}
       return huntype(fp, fpo, stderr, pname, cols, hextype,
-      		negseek ? -seekoff : seekoff);
+		negseek ? -seekoff : seekoff);
     }
 
   if (seekoff || negseek || !relseek)
     {
 #ifdef TRY_SEEK
       if (relseek)
-        e = fseek(fp, negseek ? -seekoff : seekoff, 1);
+	e = fseek(fp, negseek ? -seekoff : seekoff, 1);
       else
-        e = fseek(fp, negseek ? -seekoff : seekoff, negseek ? 2 : 0);
+	e = fseek(fp, negseek ? -seekoff : seekoff, negseek ? 2 : 0);
       if (e < 0 && negseek)
-        {
+	{
 	  fprintf(stderr, "%s: sorry cannot seek.\n", pname);
 	  return 4;
 	}
       if (e >= 0)
-        seekoff = ftell(fp);
+	seekoff = ftell(fp);
       else
 #endif
 	{
 	  long s = seekoff;
 
 	  while (s--)
-	    getc(fp);
+	    (void)getc(fp);
 	}
     }
 
@@ -589,7 +672,7 @@ char *argv[];
     {
       if (fp != stdin)
 	{
-	  fprintf(fpo, "unsigned char %s", isdigit(argv[1][0]) ? "__" : "");
+	  fprintf(fpo, "unsigned char %s", isdigit((int)argv[1][0]) ? "__" : "");
 	  for (e = 0; (c = argv[1][e]) != 0; e++)
 	    putc(isalnum(c) ? c : '_', fpo);
 	  fputs("[] = {\n", fpo);
@@ -604,11 +687,11 @@ char *argv[];
 	}
 
       if (p)
-        fputs("\n};\n"+3*(fp == stdin), fpo);
+	fputs("\n};\n"+3*(fp == stdin), fpo);
 
       if (fp != stdin)
 	{
-	  fprintf(fpo, "unsigned int %s", isdigit(argv[1][0]) ? "__" : "");
+	  fprintf(fpo, "unsigned int %s", isdigit((int)argv[1][0]) ? "__" : "");
 	  for (e = 0; (c = argv[1][e]) != 0; e++)
 	    putc(isalnum(c) ? c : '_', fpo);
 	  fprintf(fpo, "_len = %d;\n", p);
@@ -624,8 +707,8 @@ char *argv[];
       p = cols;
       while ((length < 0 || n < length) && (e = getc(fp)) != EOF)
 	{
-          putchar(hexx[(e >> 4) & 0xf]);
-          putchar(hexx[(e     ) & 0xf]);
+	  putchar(hexx[(e >> 4) & 0xf]);
+	  putchar(hexx[(e     ) & 0xf]);
 	  n++;
 	  if (!--p)
 	    {
@@ -640,9 +723,12 @@ char *argv[];
       return 0;
     }
 
-  /* hextype == HEX_NORMAL */
+  /* hextype: HEX_NORMAL or HEX_BITS */
 
-  grplen = octspergrp + octspergrp + 1;		/* chars per octet group */
+  if (hextype == HEX_NORMAL)
+    grplen = octspergrp + octspergrp + 1;	/* chars per octet group */
+  else	/* hextype == HEX_BITS */
+    grplen = 8 * octspergrp + 1;
 
   while ((length < 0 || n < length) && (e = getc(fp)) != EOF)
     {
@@ -651,13 +737,31 @@ char *argv[];
 	  sprintf(l, "%07lx: ", n + seekoff);
 	  for (c = 9; c < LLEN; l[c++] = ' ');
 	}
-      l[c = (9 + (grplen * p) / octspergrp)] = hexx[(e >> 4) & 0xf];
-      l[++c]                                 = hexx[ e       & 0xf];
-      if (ebcdic) 
-        e = (e < 64) ? '.' : etoa64[e-64];
-      l[11 + (grplen * cols - 1)/octspergrp + p] = (e > 31 && e < 127) ? e : '.';
+      if (hextype == HEX_NORMAL)
+	{
+	  l[c = (9 + (grplen * p) / octspergrp)] = hexx[(e >> 4) & 0xf];
+	  l[++c]			       = hexx[ e       & 0xf];
+	}
+      else /* hextype == HEX_BITS */
+	{
+	  int i;
+
+	  c = (9 + (grplen * p) / octspergrp) - 1;
+	  for (i = 7; i >= 0; i--)
+	    l[++c] = (e & (1 << i)) ? '1' : '0';
+	}
+      if (ebcdic)
+	e = (e < 64) ? '.' : etoa64[e-64];
+      /* When changing this update definition of LLEN above. */
+      l[11 + (grplen * cols - 1)/octspergrp + p] =
+#ifdef __MVS__
+	  (e >= 64)
+#else
+	  (e > 31 && e < 127)
+#endif
+	  ? e : '.';
       if (e)
-        nonzero++;
+	nonzero++;
       n++;
       if (++p == cols)
 	{
@@ -673,7 +777,7 @@ char *argv[];
       xxdline(fpo, l, 1);
     }
   else if (autoskip)
-    xxdline(fpo, l, -1);	/* last chance to flush out supressed lines */
+    xxdline(fpo, l, -1);	/* last chance to flush out suppressed lines */
 
   fclose(fp);
   fclose(fpo);
